@@ -16,11 +16,11 @@
 # ==============================================================================
 """An interactive NVIDIA-GPU process viewer and beyond, the one-stop solution for GPU process management."""
 
-__version__ = '1.1.2'
-__license__ = 'GPLv3'
+__version__ = '1.3.2'
+__license__ = 'GPL-3.0-only AND Apache-2.0'
 __author__ = __maintainer__ = 'Xuehai Pan'
 __email__ = 'XuehaiPan@pku.edu.cn'
-__release__ = True
+__release__ = False
 
 if not __release__:
     import os
@@ -29,7 +29,7 @@ if not __release__:
     try:
         prefix, sep, suffix = (
             subprocess.check_output(
-                ['git', 'describe', '--abbrev=7'],
+                ['git', 'describe', '--abbrev=7'],  # noqa: S603,S607
                 cwd=os.path.dirname(os.path.abspath(__file__)),
                 stderr=subprocess.DEVNULL,
                 text=True,
@@ -68,6 +68,10 @@ PYNVML_VERSION_CANDIDATES = (
     '11.515.75',
     '11.525.84',
     '11.525.112',
+    '11.525.131',
+    '12.535.77',
+    '12.535.108',
+    '12.535.133',
 )
 """The list of supported ``nvidia-ml-py`` versions.
 See also: `nvidia-ml-py's Release History <https://pypi.org/project/nvidia-ml-py/#history>`_.
